@@ -23,10 +23,12 @@ import sys
 import time
 import uuid
 
+import medusa.utils
 from medusa.cassandra_utils import Cassandra, is_node_up, wait_for_node_to_go_down
 from medusa.download import download_data
 from medusa.storage import Storage
 from medusa.verify_restore import verify_restore
+from medusa.filtering import filter_fqtns
 from medusa.network.hostname_resolver import HostnameResolver
 import medusa.config
 
@@ -69,26 +71,31 @@ def restore_node_locally(config, temp_dir, backup_name, in_place, keep_auth, see
         logging.error('No such backup')
         sys.exit(1)
 
-    fqtns_to_restore = get_fqtns_to_restore(keyspaces, tables, node_backup.manifest)
+    fqtns_to_restore, ignored_fqtns = filter_fqtns(keyspaces, tables, node_backup.manifest)
+    for fqtns in ignored_fqtns:
+        logging.info('Skipping restore of {}'.format(fqtns))
+
     if len(fqtns_to_restore) == 0:
         logging.error('There is nothing to restore')
         sys.exit(0)
 
-    cassandra = Cassandra(config.cassandra)
+    cassandra = Cassandra(config)
 
     # Download the backup
     download_dir = temp_dir / 'medusa-restore-{}'.format(uuid.uuid4())
     logging.info('Downloading data from backup to {}'.format(download_dir))
     download_data(config.storage, node_backup, fqtns_to_restore, destination=download_dir)
 
-    logging.info('Stopping Cassandra')
-    cassandra.shutdown()
-    wait_for_node_to_go_down(config, cassandra.hostname)
+    if not medusa.utils.evaluate_boolean(config.kubernetes.enabled):
+        logging.info('Stopping Cassandra')
+        cassandra.shutdown()
+        wait_for_node_to_go_down(config, cassandra.hostname)
 
     # Clean the commitlogs, the saved cache to prevent any kind of conflict
     # especially around system tables.
-    clean_path(cassandra.commit_logs_path, keep_folder=True)
-    clean_path(cassandra.saved_caches_path, keep_folder=True)
+    use_sudo = not medusa.utils.evaluate_boolean(config.kubernetes.enabled)
+    clean_path(cassandra.commit_logs_path, use_sudo, keep_folder=True)
+    clean_path(cassandra.saved_caches_path, use_sudo, keep_folder=True)
 
     # move backup data to Cassandra data directory according to system table
     logging.info('Moving backup data to Cassandra data directory')
@@ -98,7 +105,7 @@ def restore_node_locally(config, temp_dir, backup_name, in_place, keep_auth, see
         if fqtn not in fqtns_to_restore:
             logging.debug('Skipping restore for {}'.format(fqtn))
             continue
-        maybe_restore_section(section, download_dir, cassandra.root, in_place, keep_auth)
+        maybe_restore_section(section, download_dir, cassandra.root, in_place, keep_auth, use_sudo)
 
     node_fqdn = storage.config.fqdn
     token_map_file = download_dir / 'tokenmap.json'
@@ -107,26 +114,31 @@ def restore_node_locally(config, temp_dir, backup_name, in_place, keep_auth, see
         logging.debug("Parsed tokens: {}".format(tokens))
 
     # possibly wait for seeds
-    if seeds is not None:
-        wait_for_seeds(config, seeds)
-    else:
-        logging.info('No --seeds specified so we will not wait for any')
+    #
+    # In a Kubernetes deployment we can assume that seed nodes will be started first. It will
+    # handled either by the statefulset controller or by the controller of a Cassandra
+    # operator.
+    if not medusa.utils.evaluate_boolean(config.kubernetes.enabled):
+        if seeds is not None:
+            wait_for_seeds(config, seeds)
+        else:
+            logging.info('No --seeds specified so we will not wait for any')
 
-    # Start up Cassandra
-    logging.info('Starting Cassandra')
-    # restoring in place retains system.local, which has tokens in it. no need to specify extra
-    if in_place:
-        cassandra.start_with_implicit_token()
-    else:
-        cassandra.start(tokens)
+        # Start up Cassandra
+        logging.info('Starting Cassandra')
+        # restoring in place retains system.local, which has tokens in it. no need to specify extra
+        if in_place:
+            cassandra.start_with_implicit_token()
+        else:
+            cassandra.start(tokens)
 
     # Clean the restored data from local temporary folder
-    clean_path(download_dir, keep_folder=False)
+    clean_path(download_dir, use_sudo, keep_folder=False)
     return node_backup
 
 
 def restore_node_sstableloader(config, temp_dir, backup_name, in_place, keep_auth, seeds, storage, keyspaces, tables):
-    cassandra = Cassandra(config.cassandra)
+    cassandra = Cassandra(config)
     node_backup = None
     fqdns = config.storage.fqdn.split(",")
     for fqdn in fqdns:
@@ -143,7 +155,10 @@ def restore_node_sstableloader(config, temp_dir, backup_name, in_place, keep_aut
             logging.error('No such backup')
             sys.exit(1)
 
-        fqtns_to_restore = get_fqtns_to_restore(keyspaces, tables, node_backup.manifest)
+        fqtns_to_restore, ignored_fqtns = filter_fqtns(keyspaces, tables, node_backup.manifest)
+
+        for fqtns in ignored_fqtns:
+            logging.info('Skipping restore of {}'.format(fqtns))
 
         if len(fqtns_to_restore) == 0:
             logging.error('There is nothing to restore')
@@ -162,7 +177,7 @@ def restore_node_sstableloader(config, temp_dir, backup_name, in_place, keep_aut
 
 
 def invoke_sstableloader(config, download_dir, keep_auth, fqtns_to_restore, storage_port):
-    hostname_resolver = HostnameResolver(medusa.config.evaluate_boolean(config.cassandra.resolve_ip_addresses))
+    hostname_resolver = HostnameResolver(medusa.utils.evaluate_boolean(config.cassandra.resolve_ip_addresses))
     cassandra_is_ccm = int(shlex.split(config.cassandra.is_ccm)[0])
     keyspaces = os.listdir(str(download_dir))
     for keyspace in keyspaces:
@@ -175,9 +190,11 @@ def invoke_sstableloader(config, download_dir, keep_auth, fqtns_to_restore, stor
                     logging.debug('Restoring table {} with sstableloader...'.format(table))
                     cql_username = 'foo' if config.cassandra.cql_username is None else config.cassandra.cql_username
                     cql_password = 'foo' if config.cassandra.cql_password is None else config.cassandra.cql_password
-                    sstableloader_args = [config.cassandra.sstableloader_bin,
+                    sstableloader_args = [config.cassandra.sstableloader_bin.replace(
+                                          "github:apache/", "githubCOLONapacheSLASH"),
                                           '-d', hostname_resolver.resolve_fqdn() if cassandra_is_ccm == 0
                                           else '127.0.0.1',
+                                          '--conf-path', config.cassandra.config_file,
                                           '--username', cql_username,
                                           '--password', cql_password,
                                           '--no-progress',
@@ -230,7 +247,7 @@ def table_is_allowed_to_restore(keyspace, table, fqtns_to_restore):
     return True
 
 
-def clean_path(p, keep_folder=False):
+def clean_path(p, use_sudo=True, keep_folder=False):
     path = str(p)
     if p.exists() and os.path.isdir(path) and len(os.listdir(path)):
         logging.debug('Cleaning ({})'.format(path))
@@ -239,13 +256,19 @@ def clean_path(p, keep_folder=False):
             for f in os.listdir(path):
                 file_path = os.path.join(path, f)
                 logging.debug('Removing file {}'.format(file_path))
-                subprocess.check_output(['sudo', '-u', p.owner(), 'rm', '-rf', file_path])
+                if use_sudo:
+                    subprocess.check_output(['sudo', '-u', p.owner(), 'rm', '-rf', file_path])
+                else:
+                    subprocess.check_output(['rm', '-rf', file_path])
         else:
             logging.debug('Remove folder {} and content'.format(path))
-            subprocess.check_output(['sudo', '-u', p.owner(), 'rm', '-rf', path])
+            if use_sudo:
+                subprocess.check_output(['sudo', '-u', p.owner(), 'rm', '-rf', path])
+            else:
+                subprocess.check_output(['rm', '-rf', path])
 
 
-def maybe_restore_section(section, download_dir, cassandra_data_dir, in_place, keep_auth):
+def maybe_restore_section(section, download_dir, cassandra_data_dir, in_place, keep_auth, use_sudo=True):
 
     # decide whether to restore files for this table or not
 
@@ -259,7 +282,7 @@ def maybe_restore_section(section, download_dir, cassandra_data_dir, in_place, k
 
     if not in_place:
         if section['keyspace'] == 'system':
-            if section['columnfamily'].startswith('local-') or section['columnfamily'].startswith('peers-'):
+            if section['columnfamily'].startswith('local-') or section['columnfamily'].startswith('peers'):
                 restore_section = False
         if section['keyspace'] == 'system_auth' and keep_auth:
             logging.info('Keeping section {}.{} untouched'.format(section['keyspace'], section['columnfamily']))
@@ -272,22 +295,37 @@ def maybe_restore_section(section, download_dir, cassandra_data_dir, in_place, k
     # prepare the destination folder
     if dst.exists():
         logging.debug('Cleaning directory {}'.format(dst))
-        subprocess.check_output(['sudo', '-u', cassandra_data_dir.owner(),
-                                 'rm', '-rf', str(dst)])
+        if use_sudo:
+            subprocess.check_output(['sudo', '-u', cassandra_data_dir.owner(),
+                                     'rm', '-rf', str(dst)])
+        else:
+            subprocess.check_output(['rm', '-rf', str(dst)])
     else:
         logging.debug('Creating directory {}'.format(dst))
-        subprocess.check_output(['sudo', '-u', cassandra_data_dir.owner(),
-                                 'mkdir', '-p', str(cassandra_data_dir / section['keyspace'])])
+        if use_sudo:
+            subprocess.check_output(['sudo', '-u', cassandra_data_dir.owner(),
+                                     'mkdir', '-p', str(cassandra_data_dir / section['keyspace'])])
+        else:
+            subprocess.check_output(['mkdir', '-p', str(cassandra_data_dir / section['keyspace'])])
 
     if not restore_section:
         logging.debug("Skipping the actual restore of {}".format(section['columnfamily']))
         return
 
+    if not section['objects']:
+        logging.debug("Skipping the actual restore of {} - table empty".format(section['columnfamily']))
+        return
+
     # restore the table
     logging.debug('Restoring {} -> {}'.format(src, dst))
-    subprocess.check_output(['sudo', 'mv', str(src), str(dst)])
-    file_ownership = '{}:{}'.format(cassandra_data_dir.owner(), cassandra_data_dir.group())
-    subprocess.check_output(['sudo', 'chown', '-R', file_ownership, str(dst)])
+    if use_sudo:
+        subprocess.check_output(['sudo', 'mv', str(src), str(dst)])
+        file_ownership = '{}:{}'.format(cassandra_data_dir.owner(), cassandra_data_dir.group())
+        subprocess.check_output(['sudo', 'chown', '-R', file_ownership, str(dst)])
+    else:
+        subprocess.check_output(['mv', str(src), str(dst)])
+        file_ownership = '{}:{}'.format(cassandra_data_dir.owner(), cassandra_data_dir.group())
+        subprocess.check_output(['chown', '-R', file_ownership, str(dst)])
 
 
 def get_node_tokens(node_fqdn, token_map_file):
@@ -313,38 +351,3 @@ def wait_for_seeds(config, seeds):
             logging.error('Gave up waiting for seeds, aborting the restore')
             sys.exit(1)
     logging.info('At least one seed is now up')
-
-
-def get_fqtns_to_restore(keep_keyspaces, keep_tables, manifest_str):
-
-    SYSTEM_KEYSPACES = ['system', 'system_schema', 'system_auth', 'system_distributed']
-    fqtns = set()
-    manifest = json.loads(manifest_str)
-
-    for section in manifest:
-        ks = section['keyspace']
-        # in manifest, the table names have cfids, but from CLI we get it without
-        # we need to take care and use both
-        t = section['columnfamily'].split('-')[0]
-
-        fqtn = '{}.{}'.format(ks, t)
-        fqtn_with_id = '{}.{}'.format(section['keyspace'], section['columnfamily'])
-
-        # if not keyspaces / tables were specified, we keep everything
-        if len(keep_keyspaces) == 0 and len(keep_tables) == 0:
-            fqtns.add(fqtn_with_id)
-            continue
-
-        # if the whole keyspace is a keep, or a system keyspace (C* internal)
-        if ks in keep_keyspaces or ks in SYSTEM_KEYSPACES:
-            fqtns.add(fqtn_with_id)
-            continue
-
-        # if just the table is a keep
-        if fqtn in keep_tables:
-            fqtns.add(fqtn_with_id)
-            continue
-
-        logging.info('Skipping restore of {}.{}'.format(ks, t))
-
-    return fqtns
